import { defineNuxtModule, resolvePath, addPluginTemplate } from '@nuxt/kit';

function normalizeWindowsPath(input = "") {
  if (!input.includes("\\")) {
    return input;
  }
  return input.replace(/\\/g, "/");
}
const dirname = function(p) {
  return normalizeWindowsPath(p).replace(/\/$/, "").split("/").slice(0, -1).join("/") || "/";
};

function genString(input, opts = {}) {
  const str = JSON.stringify(input);
  if (!opts.singleQuotes) {
    return JSON.stringify(input);
  }
  return `'${escapeString(str)}'`;
}
const NEEDS_ESCAPE_RE = /[\\'\r\n\u2028\u2029]/;
const QUOTE_NEWLINE_RE = /(['\r\n\u2028\u2029])/g;
const BACKSLASH_RE = /\\/g;
function escapeString(id) {
  if (!id.match(NEEDS_ESCAPE_RE)) {
    return id;
  }
  return id.replace(BACKSLASH_RE, "\\\\").replace(QUOTE_NEWLINE_RE, "\\$1");
}

function wrapInDelimiters(lines, indent = "", delimiters = "{}", withComma = true) {
  if (!lines.length) {
    return delimiters;
  }
  const [start, end] = delimiters;
  return `${start}
` + lines.join(withComma ? ",\n" : "\n") + `
${indent}${end}`;
}
const VALID_IDENTIFIER_RE = /^[$_]?[\w\d]*$/;
function genObjectKey(key) {
  return key.match(VALID_IDENTIFIER_RE) ? key : genString(key);
}

function genObjectFromRaw(obj, indent = "") {
  return genObjectFromRawEntries(Object.entries(obj), indent);
}
function genArrayFromRaw(array, indent = "") {
  const newIdent = indent + "  ";
  return wrapInDelimiters(array.map((i) => `${newIdent}${genRawValue(i, newIdent)}`), indent, "[]");
}
function genObjectFromRawEntries(array, indent = "") {
  const newIdent = indent + "  ";
  return wrapInDelimiters(array.map(([key, value]) => `${newIdent}${genObjectKey(key)}: ${genRawValue(value, newIdent)}`), indent, "{}");
}
function genRawValue(value, indent = "") {
  if (typeof value === "undefined") {
    return "undefined";
  }
  if (value === null) {
    return "null";
  }
  if (Array.isArray(value)) {
    return genArrayFromRaw(value, indent);
  }
  if (value && typeof value === "object") {
    return genObjectFromRaw(value, indent);
  }
  return value.toString();
}

const fileExt = /\.s[a|c]ss$/;
function getScssTransformPlugin(sassVariables) {
  const sassImportCode = ["@import 'quasar/src/css/variables.sass'", ""];
  if (typeof sassVariables === "string") {
    sassImportCode.unshift(`@import '${sassVariables}'`);
  }
  return {
    name: "vite:quasar:scss",
    enforce: "pre",
    transform(src, id) {
      const shouldTransform = id.match(fileExt)?.[0] || false;
      if (shouldTransform) {
        const prefix = shouldTransform === ".sass" ? sassImportCode.join("\n") : sassImportCode.join(";\n");
        const useIndex = Math.max(src.lastIndexOf("@use "), src.lastIndexOf("@forward "));
        const content = {
          code: src + "\n" + prefix,
          map: null
        };
        if (useIndex === -1) {
          content.code = prefix + src;
        } else {
          const newLineIndex = src.indexOf("\n", useIndex);
          if (newLineIndex !== -1) {
            const index = newLineIndex + 1;
            content.code = src.substring(0, index) + prefix + src.substring(index);
          }
        }
        return content;
      }
      return null;
    }
  };
}

const __QUASAR_VERSION__ = `'2.7.3'`;
const module = defineNuxtModule({
  meta: {
    name: "nuxt-quasar-vite",
    configKey: "quasar",
    compatibility: {
      nuxt: "^3.0.0"
    }
  },
  defaults: {
    sassVariables: true,
    css: ["quasar/src/css/index.sass"],
    plugins: ["Notify"],
    config: {
      dark: true
    }
  },
  hooks: {
    "autoImports:sources": (presets) => {
      presets.push({
        from: "quasar/src/composables",
        imports: [
          "useQuasar",
          "useDialogPluginComponent",
          "useFormChild"
        ]
      }, {
        from: "quasar/src/utils",
        imports: [
          ["clone", "qclone"],
          ["colors", "qcolors"],
          ["copyToClipboard", "qcopyToClipboard"],
          ["createMetaMixin", "qcreateMetaMixin"],
          ["createUploaderComponent", "qcreateUploaderComponent"],
          ["date", "qdate"],
          ["debounce", "qdebounce"],
          ["dom", "qdom"],
          ["event", "qevent"],
          ["exportFile", "qexportFile"],
          ["extend", "qextend"],
          ["format", "qformat"],
          ["frameDebounce", "qframeDebounce"],
          ["getCssVar", "qgetCssVar"],
          ["noop", "qnoop"],
          ["morph", "qmorph"],
          ["openURL", "qopenURL"],
          ["patterns", "qpatterns"],
          ["scroll", "qscroll"],
          ["setCssVar", "qsetCssVar"],
          ["throttle", "qthrottle"],
          ["uid", "quid"]
        ]
      });
    },
    "components:dirs": async (dirs) => {
      const source = dirname(await resolvePath("quasar/src/components")) + "/components";
      dirs.push({
        path: source,
        transpile: true,
        watch: false,
        pattern: "*/Q*.js",
        pathPrefix: false
      });
    },
    "build:before": (_, options) => {
      if (!options.transpile.includes("quasar")) {
        options.transpile.unshift("quasar");
      }
    },
    "prepare:types": ({ references }) => {
      references.unshift({ types: "quasar" });
    }
  },
  setup: (opts, nuxt) => {
    nuxt.options.css.push(...opts.css);
    nuxt.hook("vite:extendConfig", async (config, { isServer, isClient }) => {
      const define = {
        __QUASAR_VERSION__: `${__QUASAR_VERSION__}`,
        __QUASAR_SSR__: isServer,
        __QUASAR_SSR_SERVER__: isServer,
        __QUASAR_SSR_CLIENT__: isServer && isClient,
        __QUASAR_SSR_PWA__: false
      };
      Object.assign(config.define, define);
      if (opts.sassVariables) {
        const scssPath = typeof opts.sassVariables === "string" ? await resolvePath(opts.sassVariables) : true;
        const quasarScssTransform = getScssTransformPlugin(scssPath);
        config.plugins.push(quasarScssTransform);
      }
      return config;
    });
    addPluginTemplate({
      filename: "plugins/quasarVueUse.ts",
      mode: "all",
      write: true,
      getContents: () => {
        const config = genObjectFromRawEntries(Object.entries(opts.config), "		");
        const plugins = opts.plugins.join(",");
        return `import installQ from 'quasar/src/install-quasar'
import { ${plugins} } from 'quasar/src/plugins'
import lang from 'quasar/src/lang'
import iconSet from 'quasar/src/icon-set'
import * as directives from 'quasar/src/directives'

export default defineNuxtPlugin((nuxtApp) => {
  const includes = {
    directives,
    plugins: { ${plugins} },
    config: ${config},
  }
  
  nuxtApp.vueApp.use({
    version: ${__QUASAR_VERSION__},
    install(app, opts) {
      if(process.server) {
        installQ(app, {...opts, ...includes}, nuxtApp.ssrContext)
      } else {
        installQ(app, {...opts, ...includes})
      }
    },
    lang,
    iconSet
  })
})
`;
      }
    });
  }
});

export { module as default };
